//// General Comment
module HomeNet {

	//// A Data class
	struct Data {
		Int32	a;	///< The Id
		String	b;
		Int32	c;
	};
	
	interface Test1 {
		Int32 getStatus(in Int32 a1, in Int32 a2);		///< Get the status
		Int32 getInfo(in Int32 a1, in Data a2, in Int32 a3);	/* Get Info */
	};
};
