/*******************************************************************************
 *	GenData.h	Generate Data output
 *			T.Barnaby,	BEAM Ltd,	12/9/03
 *******************************************************************************
 */
#ifndef GENDATA_HH
#define GENDATA_HH

#include <Gen.h>

class GenData : public Gen {
public:
			GenData();
			~GenData();
	BError		produce(Node* n, BString fileName);
private:
	BError		produceHeaderInt();
	BError		produceTrailerInt();
	BError		produceHeaderImp();
	BError		produceTrailerImp();
	void		produceInt(Node* n);
	void		produceImp(Node* n);

	BString		ofileName;
	FileIndent	ofileInt;
	FileIndent	ofileImp;
	BString		omodule;
	int		otypeDomain;
	int		otypeNum;
};

#endif
