/*******************************************************************************
 *	Boapns		Boap Object access protocol
 *			T.Barnaby,	BEAM Ltd,	4/3/04
 *******************************************************************************
 */
module Boapns {
	apiVersion = 1;

	class BoapEntry {
		String		name;
		String		hostName;
		List<String>	addressList;
		UInt32		port;
		UInt32		service;
	};

	interface Boapns {
		Error	getVersion(out String version);
		Error	getEntryList(out List<BoapEntry> entryList);
		Error	getEntry(in String name, out BoapEntry entry);
		Error	addEntry(in BoapEntry entry);
		Error	delEntry(in String name);
		Error	getNewName(out String name);
	};
};
