/*******************************************************************************
 *	Boapns.cc	Produced by Bidl
 *******************************************************************************
 */

#include <BoapnsD.h>

namespace Boapns {
	
	// BoapEntry Object Implementation
	BoapEntry::BoapEntry(){
	}
	BoapEntry::BoapEntry(BString pname, BString phostName, BList<BString> paddressList, UInt32 pport, UInt32 pservice){
		name = pname;
		hostName = phostName;
		addressList = paddressList;
		port = pport;
		service = pservice;
	}
}
