/*******************************************************************************
 *	TestT.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef TESTT_H
#define TESTT_H 1

#include <BString.h>
#include <BList.h>
#include <BArray.h>
#include <TestS.h>

using namespace Test;

class Obj1Server : public Test::Obj1Service {
public:
		Obj1Server(BoapServer& server, BString name);
	BError setData(Int32 a1);
	BError getData(Int32& a1);
	BError setData1(Int32 a1, Data a2);
	BError getData1(Int32& a1, Data& a2);
};

#endif
